// ==============================================================
//                 ORBITER MODULE: Dragonfly
//                  Part of the ORBITER SDK
//          Copyright (C) 2002-2003 Martin Schweiger
//                   All rights reserved
//
// Dragonfly.cpp
// Reference implementation of "Dragonfly" class space tug
//
// WARNING: This sample code will NOT create the Dragonfly DLL
// module shipped with Orbiter. The Dragonfly module is provided
// by Radu Poenaru. You should back up the dragonfly.dll before
// compiling this project!
// ==============================================================

#define STRICT

#include "Dragonfly.h"
#include "resource.h"
#include <stdio.h>
#include <math.h>

#define LOADBMP(id) (LoadBitmap (g_Param.hDLL, MAKEINTRESOURCE (id)))

struct {
	HINSTANCE hDLL;
	HFONT font[2];
	HPEN pen[1];
} g_Param;

// ==============================================================
// Specialised vessel class Dragonfly
// ==============================================================

// Constructor
Dragonfly::Dragonfly (OBJHANDLE hObj, int fmodel)
: VESSEL (hObj, fmodel)
{
	int i;

	sensormode = 0;
	cgmode = 0;
	cgofs = 0.0;
	undockmode = 0;
	undockflap = false;
	for (i = 0; i < nsurf; i++) srf[i] = 0;
}

Dragonfly::~Dragonfly ()
{
	ReleaseSurfaces ();
}

// Set vessel class parameters
void Dragonfly::SetClassCaps (FILEHANDLE cfg)
{
	int i;

	SetSize (4.0);
	SetEmptyMass (EMPTY_MASS);
	SetCrossSections (_V(23.7,22.5,17.3));
	SetPMI (_V(5.4,5.4,2.5));
	SetCameraOffset (_V(0,1.5,0));

	// ************************* propellant specs **********************************

	ph_main = CreatePropellantResource (MAX_MAIN_FUEL);

	// *********************** thruster definitions ********************************

	THRUSTER_HANDLE th_lp[5], th_rp[5], th[2];

	// thrusters in left pod
	th_lp[0] = CreateThruster (_V(-3.5,0,0), _V(1,0,0), 2*MAX_RCS_THRUST, ph_main, ISP);
	th_lp[1] = CreateThruster (_V(-2.98,0,-0.8), _V(0,0,1), MAX_RCS_THRUST, ph_main, ISP);
	th_lp[2] = CreateThruster (_V(-2.98,0,0.8), _V(0,0,-1), MAX_RCS_THRUST, ph_main, ISP);
	th_lp[3] = CreateThruster (_V(-2.98,-0.8,0), _V(0,1,0), MAX_RCS_THRUST, ph_main, ISP);
	th_lp[4] = CreateThruster (_V(-2.98,0.8,0), _V(0,-1,0), MAX_RCS_THRUST, ph_main, ISP);

	// thrusters in right pod
	th_rp[0] = CreateThruster (_V(3.5,0,0), _V(-1,0,0), 2*MAX_RCS_THRUST, ph_main, ISP);
	th_rp[1] = CreateThruster (_V(2.98,0,-0.8), _V(0,0,1), MAX_RCS_THRUST, ph_main, ISP);
	th_rp[2] = CreateThruster (_V(2.98,0,0.8), _V(0,0,-1), MAX_RCS_THRUST, ph_main, ISP);
	th_rp[3] = CreateThruster (_V(2.98,-0.8,0), _V(0,1,0), MAX_RCS_THRUST, ph_main, ISP);
	th_rp[4] = CreateThruster (_V(2.98,0.8,0), _V(0,-1,0), MAX_RCS_THRUST, ph_main, ISP);

	// thrusters in aft pod (rotational RCS)
	th_ap[0] = CreateThruster (_V(-0.8,0,-11.1), _V(1,0,0), MAX_RCS_THRUST, ph_main, ISP);
	th_ap[1] = CreateThruster (_V(0.8,0,-11.1), _V(-1,0,0), MAX_RCS_THRUST, ph_main, ISP);
	th_ap[2] = CreateThruster (_V(0,-0.8,-11.1), _V(0,1,0), MAX_RCS_THRUST, ph_main, ISP);
	th_ap[3] = CreateThruster (_V(0,0.8,-11.1), _V(0,-1,0), MAX_RCS_THRUST, ph_main, ISP);

	// exhaust definitions for left pod
	AddExhaust (th_lp[0], 1, 0.15, _V(-3.5,0.18,-0.18), _V(-1,0,0));
	AddExhaust (th_lp[0], 1, 0.15, _V(-3.5,-0.18,0.18), _V(-1,0,0));
	for (i = 1; i < 5; i++) AddExhaust (th_lp[i], 1, 0.15);

	// exhaust definitions for right pod
	AddExhaust (th_rp[0], 1, 0.15, _V(3.5,-0.18,-0.18), _V(1,0,0));
	AddExhaust (th_rp[0], 1, 0.15, _V(3.5,0.18,0.18), _V(1,0,0));
	for (i = 1; i < 5; i++) AddExhaust (th_rp[i], 1, 0.15);

	// exhaust definitions for aft pod
	for (i = 0; i < 4; i++) AddExhaust (th_ap[i], 1, 0.15);

	// thruster group definitions
	th[0] = th_lp[0];
	CreateThrusterGroup (th, 1, THGROUP_ATT_RIGHT);
	// need to add counter-balance in aft pod

	th[0] = th_rp[0];
	CreateThrusterGroup (th, 1, THGROUP_ATT_LEFT);

	th[0] = th_lp[3];
	th[1] = th_rp[3];
	CreateThrusterGroup (th, 2, THGROUP_ATT_UP);

	th[0] = th_lp[4];
	th[1] = th_rp[4];
	CreateThrusterGroup (th, 2, THGROUP_ATT_DOWN);

	th[0] = th_lp[1];
	th[1] = th_rp[1];
	CreateThrusterGroup (th, 2, THGROUP_ATT_FORWARD);

	th[0] = th_lp[2];
	th[1] = th_rp[2];
	CreateThrusterGroup (th, 2, THGROUP_ATT_BACK);

	th[0] = th_lp[3];
	th[1] = th_rp[4];
	CreateThrusterGroup (th, 2, THGROUP_ATT_BANKRIGHT);

	th[0] = th_lp[4];
	th[1] = th_rp[3];
	CreateThrusterGroup (th, 2, THGROUP_ATT_BANKLEFT);

	CreateThrusterGroup (th_ap+0, 1, THGROUP_ATT_YAWLEFT);
	CreateThrusterGroup (th_ap+1, 1, THGROUP_ATT_YAWRIGHT);
	CreateThrusterGroup (th_ap+2, 1, THGROUP_ATT_PITCHDOWN);
	CreateThrusterGroup (th_ap+3, 1, THGROUP_ATT_PITCHUP);

	// *************************** docking port ************************************

	SetDockParams (_V(0,0,3.2), _V(0,0,1), _V(0,1,0));

	// ******************************** mesh ***************************************

	AddMesh (oapiLoadMeshGlobal ("Dragonfly"));
}

void Dragonfly::ReleaseSurfaces ()
{
	for (int i = 0; i < nsurf; i++)
		if (srf[i]) {
			oapiDestroySurface (srf[i]);
			srf[i] = 0;
		}
}

void Dragonfly::InitPanel (int panel)
{
	switch (panel) {
	case 0:
		srf[0] = oapiCreateSurface (LOADBMP (IDB_DIAL1));
		srf[1] = oapiCreateSurface (LOADBMP (IDB_SWITCH1));
		srf[2] = oapiCreateSurface (LOADBMP (IDB_SWITCH2));
		break;
	}
}

void Dragonfly::DockEvent (int dock, OBJHANDLE connected)
{
	if (sensormode == 1) { // we are in remote sensing mode
		struct { OBJHANDLE hObj; int dock; } dockspec;
		dockspec.hObj = connected;
		dockspec.dock = remoteport = (connected ? 0 : -1);
		oapiBroadcastMFDMessage (MFD_DOCKING, 0, (void*)&dockspec);
	}
	oapiTriggerPanelRedrawArea (0, AID_DOCK_SENSORINFO);
}

void Dragonfly::SwitchDockref (int mode)
{
	struct { OBJHANDLE hObj; int dock; } dockspec;

	if (mode == 0) { // switch to local dock
		if (sensormode == 0) return; // nothing to do
		dockspec.hObj = GetHandle();
		dockspec.dock = 0;
	} else {
		dockspec.hObj = GetDockStatus (GetDockHandle (0));
		if (dockspec.hObj) {
			if (!sensormode || !oapiGetDockHandle (dockspec.hObj, ++remoteport))
				remoteport = 0;
		} else remoteport = -1;
		dockspec.dock = remoteport;
	}
	sensormode = mode;
	oapiBroadcastMFDMessage (MFD_DOCKING, 0, (void*)&dockspec);
	oapiTriggerPanelRedrawArea (0, AID_DOCK_SENSORINFO);
}

bool Dragonfly::SetCGMode (int mode)
{
	if (mode == cgmode) return false; // nothing to do
	cgmode = mode;
	return true;
}

bool Dragonfly::SetUndockMode (int mode)
{
	if (mode == undockmode) return false; // nothing to do
	undockmode = mode;
	return true;
}

void Dragonfly::MoveCGOfs (int dir)
{
	if (cgmode == 0) return;
	if (dir < 0) {
		if (cgofs <= 0.0) return;
		cgofs = max (0.0, cgofs - oapiGetSysStep() * 0.5);
	} else {
		if (cgofs >= 22.2) return;
		cgofs = min (22.2, cgofs + oapiGetSysStep() * 0.5);
	}
	oapiTriggerPanelRedrawArea (0, AID_DOCK_SENSORINFO);
}

void Dragonfly::Timestep (double simt)
{
	VECTOR3 cg;
	double dlevel, ratio;

	if (!cgmode) // auto
		cgofs = (GetSuperstructureCG (cg) ? cg.z : 0.0);

	if (cgofs) {
		ratio = 2.0*cgofs/(cgofs+11.1); // counter-balance level

		dlevel = GetThrusterGroupLevel (THGROUP_ATT_LEFT) - GetThrusterGroupLevel (THGROUP_ATT_RIGHT);
		if (dlevel > 0)      IncThrusterLevel_SingleStep (th_ap[0],  dlevel*ratio);
		else if (dlevel < 0) IncThrusterLevel_SingleStep (th_ap[1], -dlevel*ratio);

		dlevel = GetThrusterGroupLevel (THGROUP_ATT_DOWN) - GetThrusterGroupLevel (THGROUP_ATT_UP);
		if      (dlevel > 0) IncThrusterLevel_SingleStep (th_ap[2],  dlevel*ratio);
		else if (dlevel < 0) IncThrusterLevel_SingleStep (th_ap[3], -dlevel*ratio);
	}
}

void Dragonfly::RedrawPanel_MFDButton (SURFHANDLE surf, int mfd, int side)
{
	HDC hDC = oapiGetDC (surf);
	SelectObject (hDC, g_Param.font[1]);
	SetTextColor (hDC, RGB(20, 20, 20));
	SetTextAlign (hDC, TA_CENTER);
	SetBkMode (hDC, TRANSPARENT);
	const char *label;
	for (int bt = 0; bt < 6; bt++) {
		if (label = oapiMFDButtonLabel (mfd, bt+side*6))
			TextOut (hDC, 13, 3+34*bt, label, strlen(label));
		else break;
	}
	oapiReleaseDC (surf, hDC);
}

void Dragonfly::RedrawPanel_SensorInfo (SURFHANDLE surf)
{
	bool engaged = false;
	OBJHANDLE hObj = GetDockStatus (GetDockHandle (0));
	HDC hDC = oapiGetDC (surf);
	SelectObject (hDC, g_Param.font[1]);
	SetTextColor (hDC, RGB(0,255,0));
	SetBkMode (hDC, TRANSPARENT);
	if (!sensormode) {
		TextOut (hDC, 0, 0, "LOCAL", 5);
		engaged = (hObj != NULL);
	} else {
		TextOut (hDC, 0, 0, "REMOTE", 6);
		if (remoteport >= 0) {
			char cbuf[20];
			sprintf (cbuf, "DOCK %d", remoteport+1);
			TextOut (hDC, 0, 10, cbuf, strlen(cbuf));
			engaged = (GetDockStatus (oapiGetDockHandle (hObj, remoteport)) != NULL);
		} else 
			TextOut (hDC, 0, 10, "NO DATA", 7);
	}
	if (engaged) {
		SetTextColor (hDC, 0);
		SetBkColor (hDC, RGB(255,255,0));
		SetBkMode (hDC, OPAQUE);
		TextOut (hDC, 50, 0, "ENG", 3);
	}
	oapiReleaseDC (surf, hDC);
}

void Dragonfly::RedrawPanel_CGIndicator (SURFHANDLE surf)
{
	char cbuf[20];
	HDC hDC = oapiGetDC (surf);
	SelectObject (hDC, g_Param.font[1]);
	SetTextColor (hDC, RGB(0,255,0));
	SetBkMode (hDC, TRANSPARENT);
	sprintf (cbuf, "%0.1f m", cgofs);
	TextOut (hDC, 30, 0, cbuf, strlen (cbuf));
	int loc = 4+min ((int)(cgofs*3.784), 74);
	SelectObject (hDC, g_Param.pen[0]);
	MoveToEx (hDC, loc, 15, NULL); LineTo (hDC, loc-3, 22); LineTo (hDC, loc+3, 22); LineTo (hDC, loc, 15);
	oapiReleaseDC (surf, hDC);
}

// ==============================================================
// DLL entry point
// ==============================================================

BOOL WINAPI DllMain (HINSTANCE hModule,
					 DWORD ul_reason_for_call,
					 LPVOID lpReserved)
{
	int i;

	switch (ul_reason_for_call) {
	case DLL_PROCESS_ATTACH:
		g_Param.hDLL = hModule;
		// allocate GDI resources
		g_Param.font[0]  = CreateFont (-13, 0, 0, 0, 700, 0, 0, 0, 0, 0, 0, 0, 0, "Arial");
		g_Param.font[1]  = CreateFont (-10, 0, 0, 0, 400, 0, 0, 0, 0, 0, 0, 0, 0, "Arial");
		g_Param.pen[0] = CreatePen (PS_SOLID, 1, RGB(0,255,0));
		break;
	case DLL_PROCESS_DETACH:
		// deallocate GDI resources
		for (i = 0; i < 2; i++) DeleteObject (g_Param.font[i]);
		for (i = 0; i < 1; i++) DeleteObject (g_Param.pen[i]);
		break;
	}
	return TRUE;
}

// ==============================================================
// API interface
// ==============================================================

// Initialisation
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new Dragonfly (hvessel, flightmodel);
}

// Cleanup
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (Dragonfly*)vessel;
}

// Set the capabilities of the vessel class
DLLCLBK void ovcSetClassCaps (VESSEL *vessel, FILEHANDLE cfg)
{
	((Dragonfly*)vessel)->SetClassCaps (cfg);
}

// React to MFD mode change
DLLCLBK void ovcMFDmode (VESSEL *vessel, int mfd, int mode)
{
	switch (mfd) {
	case MFD_LEFT:
		oapiTriggerPanelRedrawArea (0, AID_MFD1_LBUTTONS);
		oapiTriggerPanelRedrawArea (0, AID_MFD1_RBUTTONS);
		break;
	}
}

// React do dock/undock events
DLLCLBK void ovcDockEvent (VESSEL *vessel, int dock, OBJHANDLE connected)
{
	((Dragonfly*)vessel)->DockEvent (dock, connected);
}

DLLCLBK void ovcTimestep (VESSEL *vessel, double simt)
{
	((Dragonfly*)vessel)->Timestep (simt);
}

// Load instrument panels
DLLCLBK bool ovcLoadPanel (VESSEL *vessel, int id)
{
	if (id != 0) return false;
	static MFDSPEC mfds_left = {{55, 20, 300, 265}, 6, 6, 41, 34};
	Dragonfly *v = (Dragonfly*)vessel;
	v->ReleaseSurfaces();

	HBITMAP hBmp = LoadBitmap (g_Param.hDLL, MAKEINTRESOURCE (IDB_PANEL1));
	oapiRegisterPanelBackground (hBmp, PANEL_ATTACH_BOTTOM|PANEL_MOVEOUT_BOTTOM, 0xffffff);
	oapiRegisterMFD (MFD_LEFT, mfds_left);
	oapiRegisterPanelArea (AID_MFD1_LBUTTONS,     _R(19,   55,  47, 244), PANEL_REDRAW_USER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_LBPRESSED, PANEL_MAP_BACKGROUND); // left button column
	oapiRegisterPanelArea (AID_MFD1_RBUTTONS,     _R(309,  55, 337, 244), PANEL_REDRAW_USER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_LBPRESSED, PANEL_MAP_BACKGROUND); // left button column
	oapiRegisterPanelArea (AID_DOCK_SENSORBUTTON, _R(366, 255, 441, 282), PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN);
	oapiRegisterPanelArea (AID_DOCK_SENSORINFO,   _R(459, 256, 529, 278), PANEL_REDRAW_USER, PANEL_MOUSE_IGNORE, PANEL_MAP_BACKGROUND);
	oapiRegisterPanelArea (AID_CG_MODE,           _R(373, 194, 395, 216), PANEL_REDRAW_MOUSE, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_RBDOWN);
	oapiRegisterPanelArea (AID_CG_MANUAL,         _R(370, 219, 397, 235), PANEL_REDRAW_MOUSE, PANEL_MOUSE_PRESSED|PANEL_MOUSE_UP);
	oapiRegisterPanelArea (AID_CG_INDICATOR,      _R(450, 200, 532, 225), PANEL_REDRAW_ALWAYS, PANEL_MOUSE_IGNORE, PANEL_MAP_BACKGROUND);
	oapiRegisterPanelArea (AID_UNDOCK_MODE,       _R(373, 143, 395, 165), PANEL_REDRAW_MOUSE, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_RBDOWN);
	oapiRegisterPanelArea (AID_UNDOCKBUTTON,      _R(465, 134, 538, 180), PANEL_REDRAW_MOUSE, PANEL_MOUSE_LBDOWN);

	v->InitPanel (id);
	return hBmp != NULL;
}

// React to panel mouse events
DLLCLBK bool ovcPanelMouseEvent (VESSEL *vessel, int id, int event, int mx, int my)
{
	Dragonfly *v = (Dragonfly*)vessel;

	switch (id) {
	case AID_MFD1_LBUTTONS:
	case AID_MFD1_RBUTTONS:
		if (my%34 < 18) {
			int bt = my/34 + (id == AID_MFD1_LBUTTONS ? 0 : 6);
			oapiProcessMFDButton (MFD_LEFT, bt, event);
			return true;
		}
		break;
	case AID_DOCK_SENSORBUTTON:
		if (mx < 34) {
			v->SwitchDockref (0);
			return true;
		} else if (mx > 41) {
			v->SwitchDockref (1);
			return true;
		}
		break;
	case AID_CG_MODE:
		return v->SetCGMode (event & PANEL_MOUSE_RBDOWN ? 1 : 0);
	case AID_CG_MANUAL:
		if (event & PANEL_MOUSE_LBPRESSED)
			v->MoveCGOfs (v->cgswitch = -1);
		else if (event & PANEL_MOUSE_RBPRESSED)
			v->MoveCGOfs (v->cgswitch = 1);
		else
			v->cgswitch = 0;
		return true;
	case AID_UNDOCK_MODE:
		return v->SetUndockMode (event & PANEL_MOUSE_RBDOWN ? 1 : 0);
	case AID_UNDOCKBUTTON:
		if (v->undockflap) {
			if (mx >= 19 && mx < 35 && my >= 11 && my < 27) {
				if (v->undockmode == 0) { // local undock
					v->Undock (0);
				} else {                  // remote undock
					OBJHANDLE mate = v->GetDockStatus (v->GetDockHandle (0));
					if (mate) oapiGetVesselInterface (mate)->Undock (ALLDOCKS, v->GetHandle());
				}
				break;
			} else if (mx >= 48 && mx < 68 && my >= 3 && my < 36) {
				v->undockflap = false;
				return true;
			}
		} else {
			if (mx >= 1 && mx < 49 && my >= 3 && my < 36) {
				v->undockflap = true;
				return true;
			}
		}
	}
	return false;
}

// React to panel redraw events
DLLCLBK bool ovcPanelRedrawEvent (VESSEL *vessel, int id, int event, SURFHANDLE surf)
{
	Dragonfly *v = (Dragonfly*)vessel;

	switch (id) {
	case AID_MFD1_LBUTTONS:
		v->RedrawPanel_MFDButton (surf, MFD_LEFT, 0);
		return true;
	case AID_MFD1_RBUTTONS:
		v->RedrawPanel_MFDButton (surf, MFD_LEFT, 1);
		return true;
	case AID_DOCK_SENSORINFO:
		v->RedrawPanel_SensorInfo (surf);
		return true;
	case AID_CG_MODE:
		oapiBlt (surf, v->srf[0], 0, 0, v->cgmode ? 22:0, 0, 22, 22);
		return true;
	case AID_CG_MANUAL:
		oapiBlt (surf, v->srf[1], 0, 0, (v->cgswitch+1)*27, 0, 27, 16);
		return true;
	case AID_CG_INDICATOR:
		v->RedrawPanel_CGIndicator (surf);
		return true;
	case AID_UNDOCK_MODE:
		oapiBlt (surf, v->srf[0], 0, 0, v->undockmode ? 22:0, 0, 22, 22);
		return true;
	case AID_UNDOCKBUTTON:
		oapiBlt (surf, v->srf[2], 0, 0, v->undockflap ? 73:0, 0, 73, 46);
		return true;
	}
	return false;
}
