// ==============================================================
//                 ORBITER MODULE: Dragonfly
//                  Part of the ORBITER SDK
//          Copyright (C) 2002-2003 Martin Schweiger
//                   All rights reserved
//
// Dragonfly.h
// Class interface of "Dragonfly" class space tug
// ==============================================================

#ifndef __DRAGONFLY_H
#define __DRAGONFLY_H

#include "orbitersdk.h"

const int nsurf = 3; // number of bitmap handles

// ==========================================================
// Some vessel class caps
// ==========================================================

const double EMPTY_MASS = 6.5e3;
// Vessel empty mass [kg]

const double MAX_MAIN_FUEL = 3.5e3;
// Max fuel capacity: main tank [kg]

const double MAX_RCS_THRUST = 1.5e3;
// Max thrust rating [N] for Reaction Control System thrusters

const double ISP = 3.8e4;
// Vacuum Isp for all thrusters [m/s]

// ==========================================================
// Interface for derived vessel class: Dragonfly
// ==========================================================

class Dragonfly: public VESSEL {
public:
	Dragonfly (OBJHANDLE hObj, int fmodel);
	~Dragonfly();
	void SetClassCaps (FILEHANDLE cfg);
	void ReleaseSurfaces();
	void InitPanel (int panel);
	void DockEvent (int dock, OBJHANDLE connected);
	void SwitchDockref (int mode);
	bool SetCGMode (int mode);
	bool SetUndockMode (int mode);
	void MoveCGOfs (int dir);
	void Timestep (double simt);
	void RedrawPanel_MFDButton (SURFHANDLE surf, int mfd, int side);
	void RedrawPanel_SensorInfo (SURFHANDLE surf);
	void RedrawPanel_CGIndicator (SURFHANDLE surf);

	SURFHANDLE srf[nsurf];      // handles for panel bitmaps
	int cgmode;                 // 0=auto, 1=manual
	int cgswitch;               // -1=left, 0=center, 1=right
	int undockmode;             // 0=local, 1=remote
	bool undockflap;            // status of undock button cover

private:
	PROPELLANT_HANDLE ph_main;
	THRUSTER_HANDLE th_ap[4];   // handles of counter-balance engines in tail fin

	int sensormode;             // docking sensor mode (local/remote)
	int remoteport;             // input reference to remote dock
	double cgofs;               // longitudinal offset of CG when attached to other object
};

#define AID_MFD1_LBUTTONS     1
#define AID_MFD1_RBUTTONS     2
#define AID_DOCK_SENSORBUTTON 3
#define AID_DOCK_SENSORINFO   4
#define AID_CG_MODE           5
#define AID_CG_MANUAL         6
#define AID_CG_INDICATOR      7
#define AID_UNDOCKBUTTON      8
#define AID_UNDOCK_MODE       9

#endif // !__DRAGONFLY_H