// ==============================================================
//                 ORBITER MODULE: FlightData
//                  Part of the ORBITER SDK
//            Copyright (C) 2003 Martin Schweiger
//                   All rights reserved
//
// FDGraph.cpp
// Flight data graph class implementation.
// ==============================================================

#include "FDGraph.h"
#include "orbitersdk.h"
#include "resource.h"

extern VESSEL *g_VESSEL;

void FlightDataGraph::AppendDataPoint ()
{
	double dp;
	float dp2[2];

	switch (dtype) {
	case 0: // altitude
		dp = g_VESSEL->GetAltitude() * 0.001;
		Graph::AppendDataPoint ((float)dp);
		return;
	case 1: // airspeed
		dp = g_VESSEL->GetAirspeed();
		Graph::AppendDataPoint ((float)dp);
		return;
	case 2: // Mach number
		dp = g_VESSEL->GetMachNumber();
		Graph::AppendDataPoint ((float)dp);
		return;
	case 3:
		dp = g_VESSEL->GetAtmTemperature();
		Graph::AppendDataPoint ((float)dp);
		return;
	case 4: // pressure
		dp2[0] = (float)(g_VESSEL->GetAtmPressure() * 0.001);
		dp2[1] = (float)(g_VESSEL->GetDynPressure() * 0.001);
		Graph::AppendDataPoints (dp2);
		return;
	case 5: // AOA
		dp2[0] = (float)(g_VESSEL->GetAOA()*DEG);
		dp2[1] = (float)(g_VESSEL->GetSlipAngle()*DEG);
		Graph::AppendDataPoints (dp2);
		return;
	case 6: // lift and drag
		dp2[0] = (float)(g_VESSEL->GetLift() * 0.001);
		dp2[1] = (float)(g_VESSEL->GetDrag() * 0.001);
		Graph::AppendDataPoints (dp2);
		return;
	case 7: // L/D
		dp = (g_VESSEL->GetDrag() ? g_VESSEL->GetLift()/g_VESSEL->GetDrag() : 0.0);
		Graph::AppendDataPoint ((float)dp);
		return;
	case 8: // Mass
		dp = g_VESSEL->GetMass();
		Graph::AppendDataPoint ((float)dp);
		return;
	}
}