// ==============================================================
// ORBITER vessel module: NASA Manned Manuveuring Unit (MMU)
// ==============================================================

#include "orbitersdk.h"

const double PI  = 3.14159265359;
const double RAD = PI/180.0;
const double slThrust = 367455;
const double vacThrust = 414340;

double GetCurrentValue(VESSEL *vessel, double sl, double vac)
{
	double c_atm=0;
	double c_value=0;
	c_atm = vessel->GetAtmPressure() / 101.4e3; 
	c_value = c_atm * (vac-sl);				
	c_value += sl;								
	return c_value;
}

void AddAttitudeJets(VESSEL *vessel)
{
	UINT atthand;
	VECTOR3 att_ref;
	VECTOR3 att_dir;
	
	//up
	att_ref= _V(-.15,-.05,-.44);
	att_dir = _V(0,0,-1);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,0,0);

	att_ref= _V(.15,-.05,-.44);
	att_dir = _V(0,0,-1);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,0,0);

	att_ref= _V(.15,.45,-.04);
	att_dir = _V(0,0,1);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,0,0);

	att_ref= _V(-.15,.45,-.04);
	att_dir = _V(0,0,1);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,0,0);

	//dn
	att_ref = _V(-.15,-.05,-.04);
	att_dir = _V(0,0,1);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,0,1);

	att_ref = _V(.15,-.05,-.04);
	att_dir = _V(0,0,1);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,0,1);
	
	att_ref = _V(.15,.45,-.44);
	att_dir = _V(0,0,-1);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,0,1);
	
	att_ref = _V(-.15,.45,-.44);
	att_dir = _V(0,0,-1);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,0,1);
	
	
	
	//right
	att_ref= _V(-.15,.45,-.44);
	att_dir = _V(0,0,-1);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,1,1);
	
	att_ref= _V(-.15,-.05,-.44);
	att_dir = _V(0,0,-1);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,1,1);
	
	att_ref= _V(.15,.45,-.04);
	att_dir = _V(0,0,1);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,1,1);
	
	att_ref= _V(.15,-.05,-.04);
	att_dir = _V(0,0,1);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,1,1);



	//left
	att_ref= _V(.15,.45,-.44);
	att_dir = _V(0,0,-1);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,1,0);
	
	att_ref= _V(.15,.05,-.44);
	att_dir = _V(0,0,-1);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,1,0);
	
	att_ref= _V(-.15,.45,-.04);
	att_dir = _V(0,0,1);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,1,0);

	att_ref= _V(-.15,-.05,-.04);
	att_dir = _V(0,0,1);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,1,0);

	//rot
	att_ref = _V(0.25,-0.05,-.35);
	att_dir = _V(1,0,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,2,0);
	att_ref = _V(-0.25,0.45,-.35);
	att_dir = _V(-1,0,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,2,0);
	att_ref = _V(0.25,0.45,-.35);
	att_dir = _V(1,0,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,2,1);
	att_ref = _V(-0.25,-0.05,-.35);
	att_dir = _V(-1,0,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,2,1);

	//tran up
	att_ref= _V(-.15,-.20,-.34);
	att_dir = _V(0,-1,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_LIN,1,0);
	att_ref= _V(.15,-.20,-.34);
	att_dir = _V(0,-1,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_LIN,1,0);
	//tran down
	att_ref= _V(-.1,.8,-.34);
	att_dir = _V(0,1,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_LIN,1,1);
	att_ref= _V(.1,.8,-.34);
	att_dir = _V(0,1,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_LIN,1,1);
	//tran left
	att_ref = _V(0.25,-0.05,-.35);
	att_dir = _V(1,0,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_LIN,0,1);
	att_ref = _V(0.25,0.45,-.35);
	att_dir = _V(1,0,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_LIN,0,1);
	//tran right
	att_ref = _V(-0.25,-0.05,-.35);
	att_dir = _V(-1,0,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_LIN,0,0);
	att_ref = _V(-0.25,0.45,-.35);
	att_dir = _V(-1,0,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_LIN,0,0);

}

void SetMMU (VESSEL *vessel)
{
// ==============================================================
// These numbers are based off of the MS Space Simulator MMU
	vessel->SetSize (1.25);
	vessel->SetCOG_elev (0.75);
	vessel->SetEmptyMass (200.0);
	vessel->SetMaxFuelMass (200.0);
	vessel->SetFuelMass(200.0);
	vessel->SetISP (350.0*9.8);
	vessel->SetMaxThrust (ENGINE_MAIN,  31.5);
	vessel->SetMaxThrust (ENGINE_RETRO, 31.5);
	vessel->SetMaxThrust (ENGINE_HOVER, 0);
	vessel->SetMaxThrust (ENGINE_ATTITUDE, 31.5);
// ==============================================================
	vessel->SetPMI (_V(.35,.13,0.35));
	vessel->SetCrossSections (_V(1.64,.77,1.64));
// ==============================================================
	vessel->SetCW (0.3, 0.3, .3, .3);
	vessel->SetRotDrag (_V(0.7,0.7,1.2));
	vessel->SetPitchMomentScale (0);
	vessel->SetBankMomentScale (0);
	vessel->SetLiftCoeffFunc (0); 
	vessel->SetDockParams (_V(0,0,.44), _V(0,0,1), _V(0,1,0));

// ==============================================================
	vessel->ClearMeshes();
	vessel->ClearExhaustRefs();
	vessel->ClearAttExhaustRefs();
	vessel->AddMesh ("mmu");
	VECTOR3 m_exhaust_pos;
	VECTOR3 m_exhaust_ref;
	m_exhaust_pos= _V(.35,.90,-.55);
	m_exhaust_ref = _V(0,0,-1);
	vessel->AddExhaustRef (EXHAUST_MAIN, m_exhaust_pos, 0.2, 0.01, &m_exhaust_ref);
	m_exhaust_pos= _V(.35,-.10,-.55);
	m_exhaust_ref = _V(0,0,-1);
	vessel->AddExhaustRef (EXHAUST_MAIN, m_exhaust_pos, 0.2, 0.01, &m_exhaust_ref);
	m_exhaust_pos= _V(-.35,0.90,-.55);
	m_exhaust_ref = _V(0,0,-1);
	vessel->AddExhaustRef (EXHAUST_MAIN, m_exhaust_pos, 0.2, 0.01, &m_exhaust_ref);
	m_exhaust_pos= _V(-.35,-0.10,-.55);
	m_exhaust_ref = _V(0,0,-1);
	vessel->AddExhaustRef (EXHAUST_MAIN, m_exhaust_pos, 0.2, 0.01, &m_exhaust_ref);

	m_exhaust_pos= _V(.37,.88,0.02);
	m_exhaust_ref = _V(0,0,1);
	vessel->AddExhaustRef (EXHAUST_RETRO, m_exhaust_pos, 0.2, 0.01, &m_exhaust_ref);
	m_exhaust_pos= _V(-.37,0.88, 0.02);
	m_exhaust_ref = _V(0,0,1);
	vessel->AddExhaustRef (EXHAUST_RETRO, m_exhaust_pos, 0.2, 0.01, &m_exhaust_ref);
	m_exhaust_pos= _V(.38,-0.40,0.02);
	m_exhaust_ref = _V(0,0,1);
	vessel->AddExhaustRef (EXHAUST_RETRO, m_exhaust_pos, 0.2, 0.01, &m_exhaust_ref);
	m_exhaust_pos= _V(-.38,-0.40,0.02);
	m_exhaust_ref = _V(0,0,1);
	vessel->AddExhaustRef (EXHAUST_RETRO, m_exhaust_pos, 0.2, 0.01, &m_exhaust_ref);
	//AddAttitudeJets(vessel);
}



// ==============================================================
// API interface
// ==============================================================



// Initialisation
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new VESSEL (hvessel, flightmodel);
}

// Cleanup
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete vessel;
}

// Keyboard interface handler
DLLCLBK int ovcConsumeKey (VESSEL *vessel, const char *keystate)
{
	OBJHANDLE hvessel;
	if (KEYMOD_SHIFT (keystate)) 
	{
		return 0; // shift combinations are reserved
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
		// insert ctrl key combinations here
	}
	else 
	{ // unmodified keys
		if (KEYDOWN (keystate, OAPI_KEY_E)) { // "End EVA"
			if (oapiAcceptDelayedKey (OAPI_KEY_E, 1.0))
			{
				char name[256];
				char vname[256];
				UINT i;
				strcpy (vname,vessel->GetName());
				for (i=0;i<strlen(vname)+1;i++)
					name[i]=0;

				for (i=0;i<strlen(vname)-4;i++)
					name[i]=vname[i];
				//strncpy (name, vname,strlen(vname)+1-4);
				hvessel=oapiGetVesselByName(name);
				if (hvessel != 0)
					oapiSetFocusObject(hvessel);
			};
			return 1;
		}
	}
	return 0;
}

// Set the capabilities of the vessel class
DLLCLBK void ovcSetClassCaps (VESSEL *vessel, FILEHANDLE cfg)
{
	SetMMU(vessel);
}


DLLCLBK void ovcTimestep (VESSEL *vessel, double simt)
{

}

DLLCLBK void ovcSetState (VESSEL *vessel, const VESSELSTATUS *status)
{
	vessel->DefSetState (status);	        // default initialisation
}